#include <stdio.h>   /* Standard input/output definitions */
#include <stdlib.h>
#include <string.h>  /* String function definitions */
#include <stdbool.h> /* Typedef for boolean */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/ioctl.h>

#include "main.h"
#include "csp.h"     /* Library for accessing the CyberPen Dockingstation */


/**
 * Written with help of http://www.easysw.com/~mike/serial/serial.html
 * and cscp_sdk
 *
 * by Thinksilicon - http://thinksilicon.de
 */
char *progname;


int main( int argc, char *argv[] ) {
	char *ser_port = "/dev/ttyS0";		// default port to open
	char *filename = "";				// Filename to write to instead of stdout

	progname = argv[ 0 ];
	
	boolean en_read = false;			// Read barcodes of device
	boolean en_clear = false;			// Clear Stick (after reading)
	boolean en_poff = false;			// Power down (after reading)

	/*-------------------------------------
	 * Handling of command-line-arguments
	 *-------------------------------------
	 */
	int c;
	extern char *optarg;
	extern int optind, optopt, opterr;
	while( ( c = getopt( argc, argv, ":p:o:rcsh" ) ) != -1 ) {
		switch( c ) {
			case 'p':
				/*
				 * Set port for communication with dockingstation
				 */
				ser_port = optarg;
				printf( "Using Serial Port %s\n", ser_port );
				break;
			case 'r':
				/*
				 * Enable read command
				 */
				en_read = true;
				break;
			case 'c':
				/*
				 * Enable clear command
				 */
				en_clear = true;
				break;
			case 's':
				/*
				 * Enable shutdown (power-off) command
				 */
				en_poff = true;
				break;
			case 'o':
				/*
				 * Set output-file - standard to stdout
				 */
				filename = optarg;
				printf( "Writing output to %s\n", filename );
				break;
			case 'h':
				printf( "Usage: %s [options]\n\
	-p Port		Set the serial port to open (default: /dev/ttyS0)\n\
	-o Filename	Set Filename to write output to (default: stdout)\n\
	-r 		Read barcodes (default: reads only user ID)\n\
	-c 		Send clear command after reading\n\
	-s 		Send power off command after reading\n\
	-h 		prints this help message and exits\n\n\
", progname );
				
				return( 0 );
				break;
			case ':':
				fprintf( stderr, "-%c missing argument\n", optopt );
				return( 1 );
				break;
			case '?':
				fprintf( stderr, "unknown argument %c\n", optopt );
				return( 1 );
				break;
		}
	}



	/*----------------------------
	 * Main part starts here
	 *----------------------------
	 */

	if( open_port( ser_port ) ) {
		init_port();

		printf( "Sending Wakeup-Command.\n" );
		char ret;
		ret = cspWakeup();
		printf( "Return Code: %i\n", ret );

		if( en_read == true ) {
			printf( "Reading barcodes.\n" );
			
			printf( "done.\n\n" );
		}
		
		if( en_clear == true ) {
			printf( "Sending Clear-Command.\n" );
			if( cspClear() == STATUS_OK ) {
				printf( "done.\n\n" );
			} else {
				fprintf( stderr, "Caught error on Clear-Command.\n" );
			}
		}
		
		if( en_poff == true ) {
			printf( "Sending Power-Off-Command.\n" );
			if( cspPOff() == STATUS_OK ) {
				printf( "done.\n\n" );
			} else {
				fprintf( stderr, "Caught error on Power-Off-Command.\n" );
			}
		}

		close_port();
		return( 0 );
	} else {
		fprintf( stderr, "Exitting...\n\n" );
		return( 1 );
	}
}
